#ifndef __LEDS_H
#define __LEDS_H



void LEDS_Exec(void);
void LEDS_GN_On(void);
void LEDS_GN_Off(void);
void LEDS_GN_Flash(unsigned int time);
void LEDS_GN_Blink_Start(unsigned int on, unsigned int off );


void LEDS_RT_On(void);
void LEDS_RT_Off(void);
void LEDS_RT_Flash(unsigned int time);
void LEDS_RT_Blink_Start(unsigned int on, unsigned int off );
void LEDS_RT_BlinkCode_Start(unsigned int blinks, unsigned int on, unsigned int off, unsigned int pause );


#endif

